
//#include "metalShaderTypes.h"

typedef struct
{
	float4 burnColor;
} fxVars;


vertex vertexOut vertexFunc(uint vertexID [[ vertex_id ]],const device txShaderVers* in[[ buffer(0) ]],constant txShaderUniforms& uniforms[[ buffer(3) ]],constant fxVars& vars[[ buffer(2)]])
{
    vertexOut out;

    out.position=in[vertexID].pos*uniforms.worldViewProj;

    out.tex1=in[vertexID].tex1;
	out.tex2=in[vertexID].tex2;
	out.texMask=in[vertexID].texMask;
    return out;
}

fragment float4 fragmentFunc(vertexOut in [[stage_in]]
							 ,texture2d<half> inputTex0 [[ texture(0) ]]
							 ,texture2d<half> inputTex1 [[ texture(1) ]]
							 ,texture2d<half> inputTexMask [[ texture(2) ]]
							 ,constant txShaderUniforms& u[[ buffer(2) ]]
							 ,constant fxVars& vars[[ buffer(1)]])
{
	  vec2 p = in.tex1;
	  
	  float T = u.transitionProgress;
	  
	  float S0 = 1.0;
	  float S1 = 250.0;
	  float S2 = 1.0;
	  
	  // 2 segments, 1/2 each
	  float Half = 0.5;
	  
	  float PixelSize = ( T < Half ) ? mix( S0, S1, T / Half ) : mix( S1, S2, (T-Half) / Half );
	  
	  vec2 D = PixelSize / float2(inputTex0.get_width(),inputTex0.get_height());
	  
	  // remap UV from 0...1 to -0.5...+0.5 to make the mosaic pattern converge torwards the image center
	  vec2 UV = ( p + vec2( -0.5 ) ) / D;
	  
	  // don't forget to remap coords back to 0...1 after ceil()
	  vec2 Coord = clamp( D * ( ceil( UV + vec2( -0.5 ) ) ) + vec2( 0.5 ), vec2( 0.0 ), vec2( 1.0 ) );

	  vec4 texColor=tex(inputTex0,Coord);

	  vec4 colorA=mix(vec4(0,0,0,0),texColor,u.mix);
	  vec4 colorB=mix(texColor,vec4(0,0,0,0),u.mix);

	float4 outColor=mix(colorB,colorA,u.transitionProgressLinear);

#ifdef USE_MASK
	outColor.a*=float(inputTexMask.sample(linearSampler,in.texMask).a);
#endif

	return outColor;
}
